@@ -121,24 +121,6 @@ class Calendar(ModelSQL, ModelView):
             ical.vevent_list.extend(ical2.vevent_list)
         return ical
 
-    @property
-    def _fbtype(self):
-        '''
-        Return the freebusy type for give transparent and status
-        '''
-        if self.transp == 'opaque':
-            if not self.status or self.status == 'confirmed':
-                fbtype = 'BUSY'
-            elif self.status == 'cancelled':
-                fbtype = 'FREE'
-            elif self.status == 'tentative':
-                fbtype = 'BUSY-TENTATIVE'
-            else:
-                fbtype = 'BUSY'
-        else:
-            fbtype = 'FREE'
-        return fbtype
-
     @classmethod
     def freebusy(cls, calendar_id, dtstart, dtend):
         '''
@@ -540,6 +522,24 @@ class Event(ModelSQL, ModelView):
     def search_calendar_field(cls, name, clause):
         return [('calendar.' + name[9:],) + tuple(clause[1:])]
 
+    @property
+    def _fbtype(self):
+        '''
+        Return the freebusy type for give transparent and status
+        '''
+        if self.transp == 'opaque':
+            if not self.status or self.status == 'confirmed':
+                fbtype = 'BUSY'
+            elif self.status == 'cancelled':
+                fbtype = 'FREE'
+            elif self.status == 'tentative':
+                fbtype = 'BUSY-TENTATIVE'
+            else:
+                fbtype = 'BUSY'
+        else:
+            fbtype = 'FREE'
+        return fbtype
+
     @classmethod
     def validate(cls, events):
         super(Event, cls).validate(events)
@@ -881,7 +881,8 @@ class Event(ModelSQL, ModelView):
             res['status'] = vevent.status.value.lower()
         else:
             res['status'] = ''
-        res['categories'] = [('remove', [c.id for c in event.categories])]
+        if event:
+            res['categories'] = [('remove', [c.id for c in event.categories])]
         if hasattr(vevent, 'categories'):
             with Transaction().set_context(active_test=False):
                 categories = Category.search([
@@ -1334,7 +1335,7 @@ class AttendeeMixin:
         res = None
         if self.attendee:
             res = vobject.base.textLineToContentLine(
-                    str(self.attendee).replace('\r\n ', ''))
+                    str(self.attendee).decode('utf-8').replace('\r\n ', ''))
         else:
             res = vobject.base.ContentLine('ATTENDEE', [], '')
 
